@@ -1,3 +1,22 @@
+#############################################################################
+#    Copyright (c) 2010-2018 Rune Haubo Bojesen Christensen
+#
+#    This file is part of the ordinal package for R (*ordinal*)
+#
+#    *ordinal* is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 2 of the License, or
+#    (at your option) any later version.
+#
+#    *ordinal* is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    A copy of the GNU General Public License is available at
+#    <https://www.r-project.org/Licenses/> and/or
+#    <http://www.gnu.org/licenses/>.
+#############################################################################
 ## This file contains:
 ## Implementation of Cumulative Link Mixed Models in clmm().
 
@@ -15,7 +34,7 @@ clmm <-
 {
 ### Extract the matched call and initial testing:
   mc <- match.call(expand.dots = FALSE)
-### FIXME: Possibly call clm() when there are no random effects?
+### OPTION: Possibly call clm() when there are no random effects?
   link <- match.arg(link)
   threshold <- match.arg(threshold)
   if(missing(formula))  stop("Model needs a formula")
@@ -26,7 +45,7 @@ clmm <-
   formulae <- clmm.formulae(formula=formula)
   ## mf, y, X, wts, off, terms:
   frames <- clmm.frames(modelcall=mc, formulae=formulae, contrasts)
-### FIXME: What should 'method="model.frame"' return? Do we want Zt
+### QUEST: What should 'method="model.frame"' return? Do we want Zt
 ### included here as well?
   if(control$method == "model.frame") return(frames)
   ## Test rank deficiency and possibly drop some parameters:
@@ -41,7 +60,10 @@ clmm <-
                offset=off, tJac=ths$tJac) })
   ## compute grouping factor list, and Zt and ST matrices:
   retrms <- getREterms(frames = frames, formulae$formula)
-### FIXME: save (the evaluated) formula in frames, so we only need the
+  ## For each r.e. term, test if Z has more columns than rows to detect
+  ## unidentifiability:
+  test_no_ranef(Zt_list=retrms$retrms, frames=frames, checkRanef=control$checkRanef)
+### OPTION: save (the evaluated) formula in frames, so we only need the
 ### frames argument to getREterms() ?
   use.ssr <- (retrms$ssr && !control$useMatrix)
 
@@ -77,7 +99,7 @@ clmm <-
           rho$ST <- par2ST(as.vector(start[[2]]), rho$ST)
       }
   }
-### FIXME: set starting values in a more elegant way.
+### OPTION: set starting values in a more elegant way.
 
   ## Set AGQ parameters:
   set.AGQ(rho, nAGQ, control, use.ssr)
@@ -107,7 +129,7 @@ clmm <-
   if(model) res$model <- frames$mf
 
   return(res)
-}
+  }
 
 clmm.formulae <- function(formula) {
     ## Evaluate the formula in the enviroment in which clmm was called
@@ -146,7 +168,7 @@ clmm.frames <- function(modelcall, formulae, contrasts) {
     fixedmf <- mf ## save call for later modification and evaluation
     fullmf <- eval(mf, envir = parent.frame(2)) ## '2' to get out of
     ## clmm.frames and clmm
-### FIXME: What if data is a matrix?
+### OPTION: Consider behavior if data is a matrix?
     fixedmf$formula <- formulae$fixedForm
     fixedmf <- eval(fixedmf, envir = parent.frame(2))
     attr(fullmf, "terms") <- attr(fixedmf, "terms")
@@ -190,11 +212,9 @@ getZt <- function(retrms) {
 getREterms <- function(frames, formula) {
 ### NOTE: Need to parse mf - not just fullmf because we need the model
 ### fits for an identifiability check below.
-    ## fullmf <- frames$mf
-### FIXME: Should we have
     fullmf <- droplevels(with(frames, mf[wts > 0, ]))
     barlist <- expandSlash(findbars(formula[[3]]))
-### FIXME: make sure 'formula' is appropriately evaluated and returned
+### NOTE: make sure 'formula' is appropriately evaluated and returned
 ### by clmm.formulae
     if(!length(barlist)) stop("No random effects terms specified in formula")
     term.names <- unlist(lapply(barlist, function(x) deparse(x)))
@@ -213,39 +233,15 @@ getREterms <- function(frames, formula) {
         Zt = do.call(rbind, lapply(seq_len(ncol(mm)), function(j) {
             Zti@x <- mm[,j]
             Zti } ))
-### FIXME: can we drop rows from Zt when g has missing values in terms
+### QUEST: can we drop rows from Zt when g has missing values in terms
 ### of the form (1 + g | f)?
         ST <- matrix(0, ncol(mm), ncol(mm),
                      dimnames = list(colnames(mm), colnames(mm)))
         list(f = ff, Zt = Zt, ST = ST)
-### FIXME: return the i'th element of Lambda here.
+### OPTION: return the i'th element of Lambda here.
     })
-    ## For each r.e. term, test if Z has more columns than rows to detect
-    ## unidentifiability:
-    for(i in seq_along(barlist)) {
-        Zti <- rel[[i]][["Zt"]]
-        if(nrow(Zti) > ncol(Zti) ||
-           (all(frames$wts == 1) && nrow(Zti) == ncol(Zti)))
-            stop(gettextf("no. random effects (=%d) >= no. observations (=%d) for term: (%s)",
-                          nrow(Zti), ncol(Zti), term.names[i]), call.=FALSE)
-    }
-    ## Test if total no. random effects >= total nobs:
     q <- sum(sapply(rel, function(x) nrow(x$Zt)))
-    if(all(frames$wts == 1) && q >= nrow(fullmf))
-        stop(gettextf("no. random effects (=%d) >= no. observations (=%d)",
-                      q, nrow(fullmf)), call.=FALSE)
-### NOTE: q > nrow(fullmf) is (sometimes) allowed if some frames$wts > 1
-###
-### NOTE: if all(frames$wts == 1) we cannot have observation-level
-### random effects so we error if nrow(Zti) >= ncol(Zti)
-###
-### FIXME: Could probably also throw an error if q >= sum(frames$wts),
-### but I am not sure about that.
-###
-### FIXME: It would be better to test the rank of the Zt matrix, but
-### also computationally more intensive.
-###
-### FIXME: If the model is nested (all gr.factors are nested), then
+### OPTION: If the model is nested (all gr.factors are nested), then
 ### order the columns of Zt, such that they come in blocks
 ### corresponding to the levels of the coarsest grouping factor. Each
 ### block of Zt-columns contain first the j'th level of the 1st gr.fac.
@@ -260,7 +256,7 @@ getREterms <- function(frames, formula) {
     nlev <- nlev[rev(order(nlev))]
     ## separate r.e. terms from the factor list:
     retrms <- lapply(rel, "[", -1)
-    names(retrms) <- NULL
+    names(retrms) <- term.names
     ## list of grouping factors:
     gfl <- lapply(rel, "[[", "f")
     ## which r.e. terms are associated with which grouping factors:
@@ -296,6 +292,42 @@ getREterms <- function(frames, formula) {
     list(retrms=retrms, gfList = gfl, dims = dims, ssr = ssr)
 }
 
+test_no_ranef <- 
+  function(Zt_list, frames, checkRanef=c("warn", "error", "message")) {
+    ## For each r.e. term, test if Z has more columns than rows to detect
+    ## unidentifiability:
+    checkfun <- switch(checkRanef,
+                       "warn" = function(...) warning(..., call.=FALSE),
+                       "error" = function(...) stop(..., call.=FALSE),
+                       "message" = message)
+    nrow_fullmf <- with(frames, nrow(mf[wts > 0, ]))
+    REterm.names <- names(Zt_list)
+    for(i in seq_along(Zt_list)) {
+      Zti <- Zt_list[[i]][["Zt"]]
+      if(nrow(Zti) > ncol(Zti) ||
+         (all(frames$wts == 1) && nrow(Zti) == ncol(Zti)))
+        checkfun(gettextf("no. random effects (=%d) >= no. observations (=%d) for term: (%s)",
+                          nrow(Zti), ncol(Zti), REterm.names[i]))
+    }
+    ## Test if total no. random effects >= total nobs:
+    q <- sum(sapply(Zt_list, function(x) nrow(x$Zt)))
+    if(all(frames$wts == 1) && q >= nrow_fullmf)
+      checkfun(gettextf("no. random effects (=%d) >= no. observations (=%d)",
+                        q, nrow_fullmf))
+    invisible(NULL)
+    ### NOTE: q > nrow(fullmf) is (sometimes) allowed if some frames$wts > 1
+    ###
+    ### NOTE: if all(frames$wts == 1) we cannot have observation-level
+    ### random effects so we error if nrow(Zti) >= ncol(Zti)
+    ###
+    ### NOTE: Could probably also throw an error if q >= sum(frames$wts),
+    ### but I am not sure about that.
+    ###
+    ### NOTE: It would be better to test the rank of the Zt matrix, but
+    ### also computationally more intensive.
+    ###
+}
+
 fe.start <- function(frames, link, threshold) {
     ## get starting values from clm:
     fit <- with(frames,
@@ -318,7 +350,7 @@ getDims <- function(frames, ths, retrms)
 rho.clm2clmm <- function(rho, retrms, ctrl)
 ### update environment, rho returned by clm.newRho().
 {
-### FIXME: write default list of control arguments?
+### OPTION: write default list of control arguments?
     ## control arguments are used when calling update.u(rho)
     rho$ctrl = ctrl
     ## compute Zt design matrix:
@@ -600,13 +632,13 @@ clmm.fit.env <-
     ## Fit the model with Laplace:
     fit <- try(nlminb(getPar.clmm(rho), function(par) getNLA(rho, par),
                       lower=lwr, control=control), silent=TRUE)
-### FIXME: Make it possible to use the ucminf optimizer with
+### OPTION: Make it possible to use the ucminf optimizer with
 ### log-transformed std-par instead.
 
     ## Check if optimizer converged without error:
     if(inherits(fit, "try-error"))
         stop("optimizer ", method, " failed to converge", call.=FALSE)
-### FIXME: Could have an argument c(warn, fail, ignore) to optionally
+### OPTION: Could have an argument c(warn, fail, ignore) to optionally
 ### return the fitted model despite the optimizer failing.
 
     ## Ensure parameters in rho are set at the optimum:
@@ -621,7 +653,7 @@ clmm.fit.env <-
                 ST = rho$ST,
                 logLik = -fit$objective,
                 dims = rho$dims,
-### FIXME: Should we evaluate hess.u(rho) to make sure rho$L contains
+### OPTION: Should we evaluate hess.u(rho) to make sure rho$L contains
 ### the right values corresponding to the optimum?
                 u = rho$u,
                 optRes = fit,
@@ -656,7 +688,7 @@ clmm.fit.env <-
         res$gradient <- grad.ctr(function(par) getNLA(rho, par, which=!bound),
                                  x=optpar)
     }
-### FIXME: We should check that the (forward) gradient for variances at the
+### OPTION: We could check that the (forward) gradient for variances at the
 ### boundary are not < -1e-5 (wrt. -logLik/nll/getNLA)
     ## Setting Niter and neval after gradient and Hessian evaluations:
     res$Niter <- rho$Niter
@@ -739,7 +771,7 @@ clmm.finalize <-
     fit$contrasts <- attr(frames$X, "contrasts")
     fit$na.action <- attr(frames$mf, "na.action")
     fit$terms <- frames$terms
-### FIXME: Should the terms object contain only the fixed effects
+### QUEST: Should the terms object contain only the fixed effects
 ### terms?
     fit$xlevels <- .getXlevels(frames$terms, frames$mf)
     fit$y.levels <- levels(frames$y)
@@ -750,8 +782,6 @@ clmm.finalize <-
         alpha <- coefficients[1:dims$nalpha]
         beta <- if(dims$nbeta > 0)
             coefficients[dims$nalpha + 1:dims$nbeta] else numeric(0)
-### QUESTION: How does lmerTest get the Hessian of varcov-parameters
-### when some are at the boundary?
         ## set various fit elements:
         edf <- dims$edf <- dims$nfepar + dims$nSTpar
         dims$nobs <- sum(frames$wts)
